//=============================================================================
// SAN_QuizBattleSystem.js
//=============================================================================
// Copyright (c) 2016 Sanshiro
//=============================================================================

/*:
 * @plugindesc クイズ戦闘システム ver0.13
 * @author サンシロ https://twitter.com/rev2nym
 * @version 0.13 2016/06/12 問題の再出題までの間隔の設定機能を追加。
 * 0.11 2016/05/01 選択肢絞り込み時に除外選択肢を選択するとエラー終了する不具合を修正。除外選択肢を表示しないよう変更。
 * 0.10 2016/05/01 アイテム選択機能、選択肢絞り込み機能を追加。
 * 0.02 2016/04/28 クイズをカテゴリー別に分類するよう変更。外部jsonファイル読込に対応。
 * 0.01 2016/04/26 試作サンプル作成。
 *
 * @help
 * クイズ戦闘システムです。
 * 本プラグインはサンシロがkotonoha*氏向けに制作したプラグインです。
 * 制作当事者の許可なく使用・改変・再配布することを禁じます。
 * 
 * @param messageBackground
 * @desc 問題文を表示するウィンドウの背景設定です。
 * 0:ウィンドウ, 1:暗くする, 2:透明
 * @default 0
 *
 * @param messagePositionType
 * @desc 問題文を表示するウィンドウの位置設定です。
 * 0:上, 1:中, 2:下
 * @default 1
 *
 * @param choiceBackground
 * @desc 選択肢を表示するウィンドウの背景設定です。
 * 0:ウィンドウ, 1:暗くする, 2:透明
 * @default 0
 *
 * @param choicePositionType
 * @desc 選択肢を表示するウィンドウの位置設定です。
 * 0:左, 1:中, 2:右
 * @default 1
 * 
 * @param choiceDefaultType
 * @desc 選択肢を表示するウィンドウのデフォルト選択肢です。
 * -1:なし, 0:選択肢#1, 0:選択肢#2, ...
 * @default 0
 * 
 * @param choiceCancelType
 * @desc 選択肢を表示するウィンドウのキャンセル選択肢です。
 * -1:禁止, 0:選択肢#1, 0:選択肢#2, ...
 * @default -1
 */

var Imported = Imported || {};
Imported.SAN_QuizBattleSystem = true;

var Sanshiro = Sanshiro || {};
Sanshiro.SAN_QuizBattleSystem = Sanshiro.SAN_QuizBattleSystem || {};
Sanshiro.SAN_QuizBattleSystem.version = '0.13';

//-----------------------------------------------------------------------------
// DataManager
//
// データマネージャ

// データベースファイル
DataManager._databaseFiles.push(
    { name: '$dataQuizRepository', src: 'QuizRepository.json' }
);

//-----------------------------------------------------------------------------
// QuizManager
//
// クイズマネージャ

function QuizManager() {}
    
// プラグインパラメータ
QuizManager._pluginParameters = PluginManager.parameters('SAN_QuizBattleSystem');

// 質問ヘッダー
QuizManager._questionHeader = "問題";

// 質問オブジェクト
QuizManager._question = {
    category:"Category",    // 質問カテゴリー
    message: "Message",     // 質問文
    choices:["Choice0", "Choice1", "Choice2", "Choice3"]    // 質問選択肢（choices[0]が正解）
}

// 出題質問カテゴリー
QuizManager._categories = [];

// 出題予約質問リスト
QuizManager._reservedQuestions = [];

// 再出題禁止質問リスト
QuizManager._intervalQuestions = [];

// 再出題禁止期間（質問数）
QuizManager._intervalLength = 0;

// 質問文
QuizManager._message = "";

// 選択肢
QuizManager._choices = [];

// 正解の選択肢
QuizManager._correctChoice = undefined;

// 選択された選択肢
QuizManager._chosenChoice = undefined;

// 禁止選択肢インデックスリスト
QuizManager._disableIndexList = [];

// 選択されたアイテムID
QuizManager._itemId = undefined;

// クイズの生成
QuizManager.createQuiz = function (categories, intervalLength) {
    this._categories = categories;
    this._intervalLength = intervalLength || 0;
    this._disableIndexList = [];
    this.initReservedQuestions();
    this.updateIntervalQuestions();
    this._question = this.pickQuestion();
    this._message  = this.makeMessage(this._question);
    this._choices  = this.makeShuffledChoices(this._question.choices);
    this._correctChoice = this._question.choices[0];
};

// 出題予約質問リストの初期化
QuizManager.initReservedQuestions = function () {
    this._reservedQuestions = $dataQuizRepository.questions.filter(function (question) {
        return this.indexOf(question.category) !== -1;
    }, this._categories);
    for (var i = 0 ;i < this._reservedQuestions.length; i++) {
        var randomIndex = Math.floor(this._reservedQuestions.length * Math.random());
        var question = this._reservedQuestions[i]
        this._reservedQuestions[i] = this._reservedQuestions[randomIndex];
        this._reservedQuestions[randomIndex] = question;
    }
};

// 再出題禁止質問リストの更新
QuizManager.updateIntervalQuestions = function () {
    if (this._intervalLength <= 0) {
        this._intervalQuestions = [];
    } else {
        var intervalLength = Math.min(this._intervalLength, this._reservedQuestions.length - 1);
        this._intervalQuestions = this._intervalQuestions.slice(-intervalLength);
    }
};

// 質問の選出
QuizManager.pickQuestion = function () {
    var question = this._reservedQuestions.shift();
    while (this._intervalQuestions.indexOf(question) !== -1) {
        question = this._reservedQuestions.shift();
    }
    this._intervalQuestions.push(question);
    return question;
};

// 質問文作成
QuizManager.makeMessage = function (question) {
    return this._questionHeader + "（" + question.category + "）" + "：\n" + question.message;
};

// 質問選択肢のシャッフル
QuizManager.makeShuffledChoices = function (choices) {
    var shuffledChoices = [];
    for (var i = 0; i < choices.length; i++) {
        shuffledChoices.push(choices[i]);
    }
    for (var i = 0; i < choices.length; i++) {
        var j = Math.floor(Math.random() * choices.length);
        var tempChoice = shuffledChoices[j];
        shuffledChoices[j] = shuffledChoices[i];
        shuffledChoices[i] = tempChoice;
    }
    return shuffledChoices;
};

// クイズの表示
QuizManager.showQuiz = function (categories) {
    if (!$gameMessage.isBusy()) {
        this._chosenChoice = undefined;
        this._itemId = undefined;
        var messagePositionType    = Number(this._pluginParameters.messagePositionType);
        var messageBackground      = Number(this._pluginParameters.messageBackground);
        var quizChoiceCancelType   = Number(this._pluginParameters.choiceCancelType);
        var quizChoiceDefaultType  = Number(this._pluginParameters.choiceDefaultType);
        var quizChoicePositionType = Number(this._pluginParameters.choicePositionType);
        var quizChoiceBackground   = Number(this._pluginParameters.choiceBackground);
        $gameMessage.setBackground(messageBackground);
        $gameMessage.setPositionType(messagePositionType);
        $gameMessage.add(this._message);
        $gameMessage.setQuizChoices(this._choices, quizChoiceDefaultType, quizChoiceCancelType);
        $gameMessage.setQuizChoiceBackground(quizChoiceBackground);
        $gameMessage.setQuizChoicePositionType(quizChoicePositionType);
    }
    return false;
}

// ランダムで不正解選択肢を禁止
QuizManager.disableRandomIncorrectChoice = function () {
    if (this._disableIndexList.length >= this._choices.length - 1) {
        return;
    }
    while (true) {
        var disableIndex = Math.floor(this._choices.length * Math.random());
        if (this._choices[disableIndex] !== this._correctChoice &&
            this._disableIndexList.indexOf(disableIndex) === -1 )
        {
            this._disableIndexList.push(disableIndex);
            break;
        }
    }
};

// 選択肢を減らす
QuizManager.decreaseChoiceTo = function (numChoices) {
    if (numChoices <= 0) {
        return;
    }
    while (this._choices.length - this._disableIndexList.length > numChoices) {
        this.disableRandomIncorrectChoice();
    }
};

// 選択肢の設定
QuizManager.setChosenChoice = function (choice) {
    this._chosenChoice = choice;
};

// 選択肢リスト
QuizManager.choices = function () {
    return this._choices;
};

// 選択アイテムIDの設定
QuizManager.setItemId = function (itemId) {
    this._itemId = itemId;
};

// アイテム選択判定
QuizManager.isItemChosen = function () {
    return this._itemId !== undefined;
};

// 回答選択判定
QuizManager.isAnswered = function () {
    return this._chosenChoice !== undefined;
};

// 回答正解判定
QuizManager.isCorrectAnswer = function () {
    return this._correctChoice === this._chosenChoice;
}

// 選択肢インデックス禁止判定
QuizManager.isDisabledIndex = function (index) {
    return this._disableIndexList.indexOf(index) !== -1;
}

//-----------------------------------------------------------------------------
// Window_QuizChoiceList
//
// クイズ選択肢ウィンドウ

function Window_QuizChoiceList() {
    this.initialize.apply(this, arguments);
}

Window_QuizChoiceList.prototype = Object.create(Window_ChoiceList.prototype);
Window_QuizChoiceList.prototype.constructor = Window_QuizChoiceList;

// オブジェクト初期化
Window_QuizChoiceList.prototype.initialize = function (messageWindow) {
    Window_ChoiceList.prototype.initialize.call(this, messageWindow);
};

// クイズアイテムウィンドウの設定
Window_QuizChoiceList.prototype.setQuizItemWindow = function (quizItemWindow) {
    this._quizItemWindow = quizItemWindow;
};

// デフォルト選択肢の選択
Window_QuizChoiceList.prototype.selectDefault = function () {
    this.select($gameMessage.quizChoiceDefaultType());
};

// ウィンドウ位置の更新
Window_QuizChoiceList.prototype.updatePlacement = function () {
    var positionType = $gameMessage.quizChoicePositionType();
    var messageY = this._messageWindow.y;
    this.width = this.windowWidth();
    this.height = this.windowHeight();
    switch (positionType) {
        case 0:
            this.x = 0;
            break;
        case 1:
            this.x = (Graphics.boxWidth - this.width) / 2;
            break;
        case 2:
            this.x = Graphics.boxWidth - this.width;
            break;
    }
    if (messageY >= Graphics.boxHeight / 2) {
        this.y = messageY - this.height;
    } else {
        this.y = messageY + this._messageWindow.height;
    }
};

// ウィンドウ背景の更新
Window_QuizChoiceList.prototype.updateBackground = function () {
    this._background = $gameMessage.quizChoiceBackground();
    this.setBackgroundType(this._background);
};

// 表示行数
Window_QuizChoiceList.prototype.numVisibleRows = function () {
    var messageY = this._messageWindow.y;
    var messageHeight = this._messageWindow.height;
    var centerY = Graphics.boxHeight / 2;
    var choices = $gameMessage.quizChoices();
    var numLines = choices.length - QuizManager._disableIndexList.length;
    var maxLines = 8;
    if (messageY < centerY && messageY + messageHeight > centerY) {
        maxLines = 4;
    }
    if (numLines > maxLines) {
        numLines = maxLines;
    }
    return numLines;
};

// 選択肢の最大幅
Window_QuizChoiceList.prototype.maxChoiceWidth = function () {//
    var maxWidth = Graphics.boxWidth;
//    var maxWidth = 96;
    var choices = $gameMessage.quizChoices();
    for (var i = 0; i < choices.length; i++) {
        var choiceWidth = this.textWidthEx(choices[i]) + this.textPadding() * 2;
        if (maxWidth < choiceWidth) {
            maxWidth = choiceWidth;
        }
    }
    return maxWidth;
};

// コマンドリスト作成
Window_QuizChoiceList.prototype.makeCommandList = function () {
    var choices = $gameMessage.quizChoices();
    for (var i = 0; i < choices.length; i++) {
        if (!QuizManager.isDisabledIndex(i)) {
            this.addCommand(choices[i], 'choice');
        }
    }
};

// 選択肢の描画
Window_QuizChoiceList.prototype.drawItem = function(index) {
    var rect = this.itemRectForText(index);
    this.changePaintOpacity(this.isCommandEnabled(index));
    this.drawTextEx(this.commandName(index), rect.x, rect.y);
};

// キャンセル許可判定
Window_QuizChoiceList.prototype.isCancelEnabled = function () {
    return $gameMessage.quizChoiceCancelType() !== -1;
};

// カーソル右移動
Window_QuizChoiceList.prototype.cursorRight = function() {
    if (this._quizItemWindow.isCursorMovable()) {
        this._quizItemWindow.select(0);
        this._quizItemWindow.unfold();
        this.deselect();
    }
};

// カーソル下移動
Window_QuizChoiceList.prototype.cursorDown = function(wrap) {
    var index = this.index();
    if (index === -1) {
        return;
    }
    var maxItems = this.maxItems();
    var maxCols = this.maxCols();
    if (index < maxItems - maxCols || (wrap && maxCols === 1)) {
        this.select((index + maxCols) % maxItems);
    }
};

// カーソル上移動
Window_QuizChoiceList.prototype.cursorUp = function(wrap) {
    var index = this.index();
    if (index === -1) {
        return;
    }
    var maxItems = this.maxItems();
    var maxCols = this.maxCols();
    if (index >= maxCols || (wrap && maxCols === 1)) {
        this.select((index - maxCols + maxItems) % maxItems);
    }
};

// 決定処理
Window_QuizChoiceList.prototype.processOk = function() {
    if (this.index() === -1){
        return;
    }
    Window_ChoiceList.prototype.processOk.call(this);
};

// タッチイベント駆動メソッド
Window_QuizChoiceList.prototype.onTouch = function(triggered) {
    Window_ChoiceList.prototype.onTouch.call(this, triggered);
    this._quizItemWindow.deselect();
    this._quizItemWindow.fold();
};

// 決定ハンドラー呼び出し
Window_QuizChoiceList.prototype.callOkHandler = function () {
    $gameMessage.onQuizChoice(this._list[this.index()].name);
    this._messageWindow.terminateMessage();
    this._quizItemWindow.deactivate();
    this._quizItemWindow.close();
    this.close();
};

// キャンセルハンドラー呼び出し
Window_QuizChoiceList.prototype.callCancelHandler = function () {
    $gameMessage.onQuizChoice(this._list[$gameMessage.quizCchoiceCancelType()].name);
    this._messageWindow.terminateMessage();
    this._quizItemWindow.deactivate();
    this._quizItemWindow.close();
    this.close();
};

//-----------------------------------------------------------------------------
// Window_QuizItem
//
// クイズアイテムウィンドウ

function Window_QuizItem() {
    this.initialize.apply(this, arguments);
}

Window_QuizItem.prototype = Object.create(Window_EventItem.prototype);
Window_QuizItem.prototype.constructor = Window_QuizItem;

// オブジェクト初期化
Window_QuizItem.prototype.initialize = function (messageWindow) {
    Window_EventItem.prototype.initialize.call(this, messageWindow);
    this.height = this.windowHeight();
    this.width = 226;
    this._fold = true;
};

// クイズ選択ウィンドウの設定
Window_QuizItem.prototype.setQuizChoiceWindow = function (quizChoiceWindow) {
    this._quizChoiceWindow = quizChoiceWindow;
};

// クイズ選択ウィンドウの高さ
Window_QuizItem.prototype.windowHeight = function () {
    return this.fittingHeight(this.numVisibleRows());
};

// 表示行数
Window_QuizItem.prototype.numVisibleRows = function () {
    if (!this._fold) {
        return this.maxItems() + 1;
    } else {
        return 1;
    }
};

// 最大列数
Window_QuizItem.prototype.maxCols = function () {
    return 1;
};

// 処理開始
Window_QuizItem.prototype.start = function () {
    this.refresh();
    this.updatePlacement();
    this.deselect();
    this.open();
    this.fold();
    this.activate();
};

// 畳み込み開始
Window_QuizItem.prototype.fold = function () {
    this._fold = true;
}

// 畳み込み解除
Window_QuizItem.prototype.unfold = function () {
    this._fold = false;
}

// アイテム表示判定
Window_QuizItem.prototype.includes = function (item) {
    var itypeId = 3;
    return DataManager.isItem(item) && item.itypeId === itypeId;
};

// 全項目描画
Window_QuizItem.prototype.drawAllItems = function () {
    var rect = this.itemRectForText(-1);
	if ($gameSwitches.value(15) === false) {
		text = "-----";
	} else if (this._data.length === 0) {
        text = "ハック選択不可";
    } else {
        text = "ハック選択可能";
    }
    this.drawText(text, rect.x, rect.y, rect.width, 'center');
    Window_EventItem.prototype.drawAllItems.call(this);
}

// 項目矩形
Window_QuizItem.prototype.itemRect = function (index) {
    var rect = Window_EventItem.prototype.itemRect.call(this, index);
    rect.y += rect.height;
    return rect;
};

// フレーム更新
Window_QuizItem.prototype.update = function () {
    Window_EventItem.prototype.update.call(this);
    this.updateFold();
};

// 畳み込み更新
Window_QuizItem.prototype.updateFold = function () {
    var diffHeight = Math.abs(this.height - this.windowHeight());
    if (this.height === this.windowHeight()) {
        return;
    } else if (this.height < this.windowHeight()) {
        this.height += Math.floor(Math.max(1, diffHeight / 2));
    } else if (this.height > this.windowHeight()) {
        this.height -= Math.floor(Math.max(1, diffHeight / 2));
    }
    if (this.height === this.windowHeight()) {
        this.createContents();
        this.drawAllItems();
    }
};

// 位置の更新
Window_QuizItem.prototype.updatePlacement = function () {
    this.x = Graphics.boxWidth - this.width;
    this.y = 0;
};

// カーソル下移動
Window_QuizItem.prototype.cursorDown = function(wrap) {
    var index = this.index();
    if (index === -1) {
        return;
    }
    var maxItems = this.maxItems();
    var maxCols = this.maxCols();
    if (index < maxItems - maxCols || (wrap && maxCols === 1)) {
        this.select((index + maxCols) % maxItems);
    }
};

// カーソル上移動
Window_QuizItem.prototype.cursorUp = function(wrap) {
    var index = this.index();
    if (index === -1) {
        return;
    }
    var maxItems = this.maxItems();
    var maxCols = this.maxCols();
    if (index >= maxCols || (wrap && maxCols === 1)) {
        this.select((index - maxCols + maxItems) % maxItems);
    }
};

// カーソル左移動
Window_QuizItem.prototype.cursorLeft = function() {
    this._quizChoiceWindow.activate();
    this._quizChoiceWindow.select(0);
    this.deselect();
    this.fold();
};

// 決定処理
Window_QuizItem.prototype.processOk = function() {
    if (this.index() === -1) {
        return;
    }
    Window_EventItem.prototype.processOk.call(this);
};

// キャンセル処理
Window_QuizItem.prototype.processCancel = function() {
    Window_EventItem.prototype.processCancel.call(this);
    this.activate();
    this.deselect();
    this.fold();
};

// タッチイベント駆動メソッド
Window_QuizItem.prototype.onTouch = function(triggered) {
    if (this.isCursorMovable()){
        Window_EventItem.prototype.onTouch.call(this, triggered);
        if (this.index() === -1) { 
            this.select(0);
        }
        this.unfold();
        this._quizChoiceWindow.deselect();
    }
};

// 決定イベント駆動メソッド
Window_QuizItem.prototype.onOk = function () {
    var item = this.item();
    QuizManager.setItemId(item ? item.id : 0);
    if (item && item.meta.SAN_QuizBattleSystem) {
        var meta = JSON.parse(item.meta.SAN_QuizBattleSystem);
        if (meta.decreaseChoiceTo !== undefined) {
            QuizManager.decreaseChoiceTo(Number(meta.decreaseChoiceTo));
        }
    }
    $gameParty.gainItem($dataItems[QuizManager._itemId], -1);
    this._messageWindow.terminateMessage();
    this._quizChoiceWindow.deactivate();
    this._quizChoiceWindow.close();
    this.deselect();
    this.close();
};

// キャンセルイベント駆動メソッド
Window_QuizItem.prototype.onCancel = function () {
    $gameVariables.setValue($gameMessage.quizItemChoiceVariableId(), 0);
    this.deselect();
    this.fold();
    this._quizChoiceWindow.select(0);
};

//-----------------------------------------------------------------------------
// Window_Message
//
// メッセージウィンドウ


// 全サブウィンドウ
Window_Message.prototype.subWindows = function () {
    return [this._goldWindow, this._choiceWindow,
            this._numberWindow, this._itemWindow,
            this._quizChoiceWindow, this._quizItemWindow];
};

// サブウィンドウ作成
Sanshiro.SAN_QuizBattleSystem.Window_Message_createSubWindows = Window_Message.prototype.createSubWindows;
Window_Message.prototype.createSubWindows = function () {
    Sanshiro.SAN_QuizBattleSystem.Window_Message_createSubWindows.call(this);
    this._quizChoiceWindow = new Window_QuizChoiceList(this);
    this._quizItemWindow = new Window_QuizItem(this);
    this._quizChoiceWindow.setQuizItemWindow(this._quizItemWindow);
    this._quizItemWindow.setQuizChoiceWindow(this._quizChoiceWindow);
};


// サブウィンドウアクティブ判定
Window_Message.prototype.isAnySubWindowActive = function() {
    return (this._choiceWindow.active ||
            this._numberWindow.active ||
            this._itemWindow.active ||
            this._quizChoiceWindow.active ||
            this._quizItemWindow.active);
};

// 入力処理の開始
Window_Message.prototype.startInput = function () {
    if ($gameMessage.isChoice()) {
        this._choiceWindow.start();
        return true;
    } else if ($gameMessage.isQuizChoice()) {
        this._quizItemWindow.start();
        this._quizChoiceWindow.start();
        return true
    } else if ($gameMessage.isNumberInput()) {
        this._numberWindow.start();
        return true;
    } else if ($gameMessage.isItemChoice()) {
        this._itemWindow.start();
        return true;
    } else {
        return false;
    }
};

//-----------------------------------------------------------------------------
// Game_Message
//
// メッセージクラス

Sanshiro.SAN_QuizBattleSystem.Game_Message_clear = Game_Message.prototype.clear;
Game_Message.prototype.clear = function () {
    Sanshiro.SAN_QuizBattleSystem.Game_Message_clear.call(this);
    this._quizChoices = [];
    this._quizChoiceDefaultType = 0;
    this._quizChoiceCancelType = 0;
    this._quizChoiceBackground = 0;
    this._quizChoicePositionType = 2;
    this._quizChoiceCallback = null;
    this._quizItemChoiceVariableId = 0;
    this._quizItemChoiceItypeId = 0;
};

// クイズ選択肢
Game_Message.prototype.quizChoices = function () {
    return this._quizChoices;
};

// クイズ選択デフォルト選択肢
Game_Message.prototype.quizChoiceDefaultType = function () {
    return this._quizChoiceDefaultType;
};

// クイズ選択キャンセル選択肢
Game_Message.prototype.quizChoiceCancelType = function () {
    return this._quizChoiceCancelType;
};

// クイズ選択ウィンドウ背景
Game_Message.prototype.quizChoiceBackground = function () {
    return this._quizChoiceBackground;
};

// クイズ選択ウィンドウ位置タイプ
Game_Message.prototype.quizChoicePositionType = function () {
    return this._quizChoicePositionType;
};

// クイズアイテムID格納変数ID
Game_Message.prototype.quizItemChoiceVariableId = function () {
    return this._itemChoiceVariableId;
};

// クイズアイテムタイプID
Game_Message.prototype.quizItemChoiceItypeId = function () {
    return this._itemChoiceItypeId;
};

// クイズ選択肢設定
Game_Message.prototype.setQuizChoices = function (choices, defaultType, cancelType) {
    this._quizChoices = choices;
    this._quizChoiceDefaultType = defaultType;
    this._quizChoiceCancelType = cancelType;
};

// クイズ選択ウィンドウ背景設定
Game_Message.prototype.setQuizChoiceBackground = function (background) {
    this._quizChoiceBackground = background;
};

// クイズ選択ウィンドウ位置設定
Game_Message.prototype.setQuizChoicePositionType = function (positionType) {
    this._quizChoicePositionType = positionType;
};

// クイズ選択コールバックメソッド設定
Game_Message.prototype.setQuizChoiceCallback = function (callback) {
    this._quizChoiceCallback = callback;
};

// クイズ選択イベント起動メソッド
Game_Message.prototype.onQuizChoice = function (choice) {
    QuizManager.setChosenChoice(choice);
};

// クイズ選択有無判定
Game_Message.prototype.isQuizChoice = function () {
    return this._quizChoices.length > 0;
};

// メッセージビジー判定
Sanshiro.SAN_QuizBattleSystem.Game_Message_isBusy = Game_Message.prototype.isBusy;
Game_Message.prototype.isBusy = function () {
    var isBusy = Sanshiro.SAN_QuizBattleSystem.Game_Message_isBusy.call(this);
    return (isBusy || this.isQuizChoice());
};
